#!/bin/bash
# -----------------------------------------------------------------------------
# Installation script for gendist. This script creates a directory-structure
# for your own distribution.
#
#
# $Author: Bablokb $
# $Revision: 1.18 $
#
# License: GPL2 (see COPYING)
# -----------------------------------------------------------------------------

setDefaults() {
  if [ -z "$GENDIST_HOME" ]; then
     cd `dirname $0`/..
     GENDIST_HOME=`pwd`
     cd -
  fi
  export GENDIST_HOME
  . $GENDIST_HOME/version.inc
  export version=$DIST_MAJOR.$DIST_MINOR.$DIST_PL
  useGpl=0
  copyDoc=0
  prefix=`pwd`
  distName="missing"
  types="missing"
  keepFiles=1
  allTypes="syslinux"
  for file in $GENDIST_HOME/template/media/Makefile.*; do
    [ "$file" = "$GENDIST_HOME/template/media/Makefile.syslinux" ] && continue
    if [ -f $file ]; then
      file=`basename $file`
      allTypes="$allTypes ${file#Makefile.}"
    fi
  done
  allTypes=${allTypes# }
}

# usage notes -----------------------------------------------------------------

usage() {
  echo -en "usage: $0 -n name -t types [-p prefix] [-ogdv]\n" \
           "Create build-system for a Linux-distribution\n\n" \
           "    -n name:   name of distribution\n" \
           "    -t types:  blank separated list of supported types\n" \
           "    -p prefix: installation directory\n" \
           "               (will install into prefix/name, default: .)\n" \
           "    -o:        overwrite existing files\n" \
           "    -g:        copy GPL to target directory\n" \
           "    -d:        copy GENDIST-documentation to target directory\n" \
           "    -v:        show version number\n\n" \
           "Supported types:\n" \
           "    all $allTypes\n"
  exit 3
}

# version information ----------------------------------------------------------

showVersion() {
  echo -en "GENDIST version $version\n\n" \
           "Copyright (C) 2002 Bernhard Bablok, mail@bablokb.de\n" \
           "This is free software; see the source for copying conditions.\n" \
           "There is NO warranty; not even for MERCHANTABILITY or\n" \
           "FITNESS FOR A PARTICULAR PURPOSE.\n"
 exit 3
}

# create file from template ---------------------------------------------------

createFile() {
  targetFile=$targetDir/$1
  new=""
  if [ -d $GENDIST_HOME/template/$1 ]; then
    echo "creating directory $targetFile"
    mkdir -p $targetFile
  else
    [ $keepFiles -eq 1 -a -f $targetFile ] && new=".new"
    [ $keepFiles -eq 0 -a -f $targetFile ] && mv -f $targetFile $targetFile.old
    echo "creating file $targetFile$new"
    bash $GENDIST_HOME/template/$1 "$distName" "$types" > $targetFile$new
    [ $? -gt 0 ] && rm -f $targetFile$new
    pruneFile
  fi
}

# parse arguments and set variables -------------------------------------------

parseArguments() {

  while getopts ":n:t:p:ogdv" opt; do
    case $opt in
      n) distName=$OPTARG;;
      t) types=$OPTARG;;
      p) prefix=$OPTARG;;
      o) keepFiles=0;;
      g) useGpl=1;;
      d) copyDoc=1;;
      v) showVersion;;
      *) usage;;
    esac
  done
  targetDir=$prefix/$distName
  if [ "$distName" = "missing" ]; then
    echo "error: name of distribution not defined"
    usage
  fi
  if [ "$types" = "missing" ]; then
    echo "error: supported types of distribution not defined" 
    usage
  elif [ "$types" = "all" ]; then
    types=$allTypes
  fi
  for t in $types; do
    if [ ! -e $GENDIST_HOME/template/media/Makefile.$t ]; then
      echo "error: $t is not a valid type"
      usage
  fi
  done
}

# remove files which didn't change --------------------------------------------

pruneFile() {
  adminFiles="ChangeLog NEWS README INSTALL version.inc \
              src/isolinux/bootinfo.in src/isolinux/isolinux.cfg.in \
              src/lilo.conf.in src/lilo/message.in \
              src/syslinux/bootinfo.in src/syslinux/syslinux.cfg.in"

  for f in $adminFiles; do
    [ "$targetDir/$f" != "$targetFile" ] && continue
    if [ -f $targetFile -a -f $targetFile".new" ]; then
      rm -f $targetFile".new"
    elif [ -f $targetFile -a -f $targetFile".old" ]; then
      mv -f $targetFile".old" $targetFile
    fi
    return
  done

  if [ -f $targetFile -a -f $targetFile".new" ]; then
    cmp -s $targetFile $targetFile".new"
    [ $? -eq 0 ] && rm -f $targetFile".new"
  elif [ -f $targetFile -a -f $targetFile".old" ]; then
    cmp -s $targetFile $targetFile".old"
    [ $? -eq 0 ] && mv -f $targetFile".old" $targetFile
  fi
}

# delete any empty directories ------------------------------------------------

pruneEmptyDirs() {
  local dirList="syslinux isolinux lilo"
  for d in $dirList; do
    [ `ls -A1 $targetDir/src/$d | wc -l` -eq 0 ] && rmdir $targetDir/src/$d
  done
}


# copy file COPYING to target directory ---------------------------------------

copyGpl() {
  if [ $useGpl -eq 1 ]; then
    if [ ! -f $targetDir/COPYING ]; then
      echo "creating file $targetDir/COPYING"
      cp $GENDIST_HOME/COPYING $targetDir
    fi
  fi
}

# copy GENDIST documentation to target directory ------------------------------

copyDocu() {
  if [ $copyDoc -eq 1 ]; then
    echo "copying GENDIST-documentation to $targetDir/doc/gendist"
      mkdir -p $targetDir/doc/gendist/images
      cp -auv $GENDIST_HOME/doc/*.html $targetDir/doc/gendist
      cp -auv $GENDIST_HOME/doc/images/* $targetDir/doc/gendist/images
  fi
}

# create GENDIST-version file in target directory ------------------------------

setVersion() {
  echo "creating file $targetDir/gendist.inc"
  echo "GENDIST_VERSION=$version" > $targetDir/gendist.inc
}

# main program ----------------------------------------------------------------

setDefaults
parseArguments "$@"
mkdir -p $targetDir
for file in \
         `find $GENDIST_HOME/template -path "*CVS" -prune -o -printf "%P "`; do
  createFile $file
done
pruneEmptyDirs
copyGpl
copyDocu
setVersion
exit 0
